#!/usr/bin/python
#  MEMU Launcer GUI
#
#  WJB   30/ 1/14 First draft.
#
try:
    from tkinter import *                 # Python 3.x import
    from tkinter.filedialog import *
except ImportError:
    from Tkinter import *                 # Python 2.x import
    from tkFileDialog import *
import subprocess
import os
import sys

class Launcher:
    def __init__ (self, args):
        self.args = args
        self.pdir = self.GetProgDir ()
        self.cfg  = self.GetConfigFile ()
        self.root = Tk ()
        self.root.title ('MEMU Launcher')
        r = 0
        Label (self.root, text='Emulation').grid (row=r, column=0)
        self.em_mode = StringVar ()
        OptionMenu (self.root, self.em_mode, 'MTX', 'SDX', 'CPM').grid (row=r, column=1, sticky='W')
        self.em_mode.set ('MTX')
        r += 1
        Label (self.root, text='Memory (KB)').grid (row=r, column=0, sticky='E')
        self.mem = StringVar ()
        Spinbox (self.root, textvariable=self.mem, from_=32, to=512, increment=32).grid (row=r, column=1, sticky='EW')
        r += 1
        Label (self.root, text='Video size').grid (row=r, column=0, sticky='E')
        self.vid = StringVar ()
        Spinbox (self.root, textvariable=self.vid, from_=0, to=5).grid (row=r, column=1, sticky='EW')
        r += 1
        Label (self.root, text='Monitor size').grid (row=r, column=0, sticky='E')
        self.mon = StringVar ()
        Spinbox (self.root, textvariable=self.mon, from_=0, to=5).grid (row=r, column=1, sticky='EW')
        r += 1
        Label (self.root, text='Mono Monitor').grid (row=r, column=0, sticky='E')
        self.mono = IntVar ()
        Checkbutton (variable=self.mono).grid (row=r, column=1, sticky='W')
        r += 1
        Label (self.root, text='Text Monitor').grid (row=r, column=0, sticky='E')
        f1 = Frame (self.root)
        f1.grid (row=r, column=1, sticky='W')
        self.tmode = IntVar ()
        Radiobutton (f1, text='None', variable=self.tmode, value=0).pack (side=LEFT)
        Radiobutton (f1, text='TH', variable=self.tmode, value=1).pack (side=LEFT)
        Radiobutton (f1, text='Console', variable=self.tmode, value=2).pack (side=LEFT)
        r += 1
        Label (self.root, text='Sound').grid (row=r,column=0, sticky='E')
        self.snd = IntVar ()
        Checkbutton (variable=self.snd).grid (row=r, column=1, sticky='W')
        r += 1
        Label (self.root, text='Keyboard Remap').grid (row=r, column=0, sticky='E')
        self.kbd = IntVar ()
        Checkbutton (variable=self.kbd).grid (row=r, column=1, sticky='W')
        r += 1
        Label (self.root, text='Tape Directory').grid (row=r, column=0, sticky='E')
        f2 = Frame (self.root)
        f2.grid (row=r, column=1, sticky='EW')
        Button (f2, text='...', command=self.BrowseTape).pack (side=RIGHT)
        self.tape = StringVar ()
        Entry (f2, textvariable=self.tape).pack (side=LEFT, fill=X, expand=1)
        r += 1
        Label (self.root, text='Drive A Directory').grid (row=r, column=0, sticky='E')
        f3 = Frame (self.root)
        f3.grid (row=r, column=1, sticky='EW')
        Button (f3, text='...', command=self.BrowseDriveA).pack (side=RIGHT)
        self.drivea = StringVar ()
        Entry (f3, textvariable=self.drivea).pack (side=LEFT, fill=X, expand=1)
        r += 1
        Label (self.root, text='Drive A Invert Case').grid (row=r, column=0, sticky='E')
        self.daic = IntVar ()
        Checkbutton (variable=self.daic).grid (row=r, column=1, sticky='W')
        r += 1
        Label (self.root, text='Run File').grid (row=r, column=0, sticky='E')
        f4 = Frame (self.root)
        f4.grid (row=r, column=1, sticky='EW')
        Button (f4, text='...', command=self.BrowseRun).pack (side=RIGHT)
        self.run = StringVar ()
        Entry (f4, textvariable=self.run).pack (side=LEFT, fill=X, expand=1)
        r += 1
        Button (self.root, text='Cancel', command=self.Quit).grid (row=r, column=0)
        Button (self.root, text='OK', command=self.Launch).grid (row=r, column=1)
        self.ReadCfg ()
        if ( ( self.vid.get () == '0' ) and ( self.mon.get () == '0' ) and
             ( self.tmode.get () == 0 ) ):
            self.vid.set ('1')
        self.root.mainloop ()

    def GetProgDir (self):
        (pdir, name) = os.path.split (self.args[0])
        if ( pdir ):
            return os.path.abspath (pdir)
        for pdir in os.get_exec_path ():
            if ( os.path.exists (os.path.join (pdir, name)) ):
                return os.path.abspath (pdir)
        return None

    def GetConfigFile (self):
        name, ext = os.path.splitext (os.path.basename (self.args[0]))
        name += '.cfg'
        if ( os.path.exists (name) ):
            return (os.path.abspath (name))
        if ( self.pdir ):
            return os.path.join (self.pdir, name)
        return None

    def BrowseTape (self):
        dir = askdirectory ()
        if ( dir ):
            self.tape.set (dir)

    def BrowseDriveA (self):
        dir = askdirectory ()
        if ( dir ):
            self.drivea.set (dir)

    def BrowseRun (self):
        defdir = self.tape.get ()
        if ( not defdir ):
            defdir = self.drivea.get ()
            if ( not defdir ):
               defdir = "."
        fn = askopenfilename (initialdir=defdir,
                              filetypes=[('Tape files', '*.mtx'),
                                         ('RUN files', '*.run'),
                                         ('CPM files', '*.com')]);
        if ( fn ):
            if ( fn[-4:].lower () == '.mtx' ):
                tape_dn = self.tape.get ()
                if ( tape_dn ):
                   fn = os.path.relpath (fn, tape_dn)
                else:
                   (tape_dn, fn) = os.path.split (fn)
                   self.tape.set (tape_dn)
            elif ( fn[-4:].lower () == '.com' ):
                a_dn = self.tape.get ()
                if ( not a_dn ):
                   a_dn = os.path.dirname (fn)
                   self.drivea.set (a_dn)
            self.run.set (fn)

    def WriteCfg (self, margs, nswitch):
        if ( self.cfg ):
           f = open (self.cfg, 'w')
           f.write (margs[1])
           for ma in margs[2:nswitch]:
               if ( ma[0] == '-' ):
                   f.write ('\n' + ma)
               elif ( ma.find (' ') >= 0 ):
                   f.write (' "' + ma + '"')
               else:
                   f.write (' ' + ma)
           f.write ('\n')
           if ( len (margs) > nswitch ):
               if ( margs[nswitch].find (' ') >= 0 ):
                   f.write ('"' + margs[nswitch] + '"')
               else:
                   fwrite (margs[nswitch])
               for ma in margs[nswitch+1:]:
                   if ( ma.find (' ') >= 0 ):
                       f.write (' "' + ma + '"')
                   else:
                       f.write (' ' + ma)
               f.write ('\n')
           f.close ()

    def Parse (self, text):
        margs = []
        even = True
        for part in text.split ('"'):
            if ( even ):
                margs.extend (part.split ())
            else:
                margs.append (part)
            even = not even
        # print margs
        return margs

    def ReadCfg (self):
        if ( ( self.cfg ) and ( os.path.exists (self.cfg) ) ):
            f = open (self.cfg, 'r')
            margs = self.Parse (f.read ())
            f.close ()
            i = 0
            while ( i < len (margs) ):
                if ( margs[i] == '-sdx' ):
                    self.em_mode.set ('SDX')
                elif ( margs[i] == '-cpm' ):
                    self.em_mode.set ('CPM')
                elif ( margs[i] == '-mem-blocks' ):
                    if ( i+1 < len (margs) ):
                        self.mem.set (str (16 * int (margs[i+1])))
                        i += 1
                elif ( margs[i] == '-vid-win' ):
                    if ( self.vid.get () == '0' ):
                        self.vid.set ('1')
                elif ( margs[i] == '-vid-win-big' ):
                    self.vid.set (str (int (self.vid.get ()) + 1))
                elif ( margs[i] == '-mon-win' ):
                    if ( self.mon.get () == '0' ):
                        self.mon.set ('1')
                elif ( margs[i] == '-mon-win-big' ):
                    self.mon.set (str (int (self.mon.get ()) + 1))
                elif ( margs[i] == '-mon-win-mono' ):
                    self.mono.set (1)
                elif ( margs[i] == '-mon-th' ):
                    self.tm_vat.set (1)
                elif ( margs[i] == '-mon-console' ):
                    self.tmode.set (2)
                elif ( margs[i] == '-snd-portaudio' ):
                    self.snd.set (1)
                elif ( margs[i] == '-kbd-remap' ):
                    self.kbd.set (1)
                elif ( margs[i] == '-tape-dir' ):
                    if ( i+1 < len (margs) ):
                        self.tape.set (margs[i+1])
                        i += 1
                elif ( margs[i] == '-cpm-drive-a' ):
                    if ( i+1 < len (margs) ):
                        self.drivea.set (margs[i+1])
                        i += 1
                elif ( margs[i] == '-cpm-invert-case' ):
                    self.daic.set (1)
                elif (margs[i][0] != '-' ):
                    self.run.set (margs[i])
                i += 1

    def Launch (self):
        if ( len (self.args) > 1 ):
            margs = self.args[1:]
        else:
            if ( os.path.exists (os.path.join (self.pdir, 'memu.exe')) ):
                margs = [os.path.join (self.pdir, 'memu.exe')]
            elif ( os.path.exists (os.path.join (self.pdir, 'memu')) ):
                margs = [os.path.join (self.pdir, 'memu')]
            else:
                margs = ['memu']
        if ( self.em_mode.get () == 'SDX' ):
            margs.append ('-sdx')
        elif ( self.em_mode.get () == 'CPM' ):
            margs.append ('-cpm')
        margs.extend (['-mem-blocks', '%d' % ( int (self.mem.get ()) / 16 )])
        vsize = int (self.vid.get ())
        if ( vsize > 0 ):
            margs.append ('-vid-win')
            for i in range (vsize -1):
                margs.append ('-vid-win-big')
        msize = int (self.mon.get ())
        if ( msize > 0 ):
            margs.append ('-mon-win')
            for i in range (msize - 1):
                margs.append ('-mon-win-big')
        if ( self.mono.get () ):
            margs.append ('-mon-win-mono')
        tm = self.tmode.get ()
        if ( tm == 1 ):
            margs.append ('-mon-th')
        elif ( tm == 2 ):
            margs.append ('-mon-console')
        if ( self.snd.get () ):
            margs.append ('-snd-portaudio')
        if ( self.kbd.get () ):
            margs.append ('-kbd-remap')
        tape_dn = self.tape.get ()
        if ( tape_dn ):
            margs.extend (['-tape-dir', tape_dn])
        drivea_dn = self.drivea.get ()
        if ( drivea_dn ):
            margs.extend (['-cpm-drive-a', drivea_dn])
        if ( self.daic.get () ):
            margs.append ('-cpm-invert-case')
        nswitch = len (margs)
        run_fn = self.run.get ()
        if ( run_fn ):
            if ( ( tape_dn ) and ( run_fn[-4:].lower () == '.mtx' ) ):
                run_dn = os.path.dirname (run_fn)
                if ( run_dn ):
                    run_fn = os.path.relpath (run_fn, tape_dn)
            margs.append (run_fn)
        # print (margs)
        self.WriteCfg (margs, nswitch)
        subprocess.Popen (margs)
        self.root.destroy ()

    def Quit (self):
        self.root.destroy ()

Launcher (sys.argv)
